# vim:set et ts=4 sts=4:
# -*- coding: utf-8 -*-
#
# ibus-libpinyin - Intelligent Pinyin engine based on libpinyin for IBus
#
# Copyright (c) 2024 Peng Wu <alexepico@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
import xml.dom.minidom
import codecs

from gi import require_version as gi_require_version
gi_require_version('GLib', '2.0')
gi_require_version('Gio', '2.0')

from gi.repository import GLib
from gi.repository import Gio

pkgdatadir = os.getenv("IBUS_PKGDATADIR") or "."

def save_layout():
    # assume the name and layout tag has the same order,
    # save both libpinyin and libbopomofo value here.
    system_config = os.path.join(pkgdatadir, 'default.xml')

    dom = xml.dom.minidom.parse(system_config)
    names = dom.getElementsByTagName('name')
    layouts = dom.getElementsByTagName('layout')
    assert len(names) == len(layouts)
    for i, name in enumerate(names):
        engine = name.childNodes[0].data
        config_namespace = "com.github.libpinyin.ibus-libpinyin." + engine
        config = Gio.Settings.new(config_namespace)
        var = config.get_value("keyboard-layout")
        assert 's' == var.get_type_string()
        layout = var.get_string()
        layouts[i].childNodes[0].data = layout

    user_config = os.path.join(GLib.get_user_config_dir(),
                               'ibus', 'libpinyin', 'engines.xml')
    dir = os.path.dirname(user_config)
    os.path.exists(dir) or os.makedirs(dir, 0o700)
    # io.open() causes TypeError for unicode.
    f = codecs.open(user_config, 'w', 'utf-8')
    dom.writexml(f, '', '', '', 'utf-8')
    f.close()
    os.chmod(user_config, 0o600)

def resync_engine_file():
    user_config = os.path.join(GLib.get_user_config_dir(),
                               'ibus', 'libpinyin', 'engines.xml')
    system_config = os.path.join(pkgdatadir, 'default.xml')
    if not os.path.exists(user_config):
        return
    if not os.path.exists(system_config):
        os.unlink(user_config)
        return

    # path.getmtime depends on the build time rather than install time.
    def __get_engine_file_version(engine_file):
        version_str = ''
        dom = xml.dom.minidom.parse(engine_file)
        # there are two versions in the engines.xml, only use the first one
        elements = dom.getElementsByTagName('version')
        nodes = []
        if len(elements) > 0:
            nodes = elements[0].childNodes
        if len(nodes) > 0:
            version_str = nodes[0].data
        if version_str != '':
            version_str = version_str.strip()
        return version_str

    user_config_version = __get_engine_file_version(user_config)
    system_config_version = __get_engine_file_version(system_config)
    if system_config_version != user_config_version:
        # generate the user config from gsettings
        save_layout()



# vim:set et ts=4 sts=4:
#
# ibus-libpinyin - Intelligent Pinyin engine based on libpinyin for IBus
#
# Copyright (c) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
# Copyright (c) 2013 Peng Wu <alexepico@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gettext

gettext.install("ibus-libpinyin")

def get_version():
    return "1.16.5"

def get_longname():
    return _("Intelligent Pinyin")

def support_cloud_input():
    return True

def enable_lua_extension():
    return True

def enable_table_mode():
    return True

def enable_english_input_mode():
    return True



# vim:set et sts=4 sw=4:
#
# ibus-libpinyin - Intelligent Pinyin engine based on libpinyin for IBus
#
# Copyright (c) 2007-2014 Peng Huang <shawn.p.huang@gmail.com>
# Copyright (c) 2007-2014 Red Hat, Inc.
# Copyright (c) 2014 Peng Wu <alexepico@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
# USA

__all__ = (
    "KeyboardShortcutSelection",
    "KeyboardShortcutSelectionDialog",
);

import gettext
import locale
import os

from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import Gtk
from gi.repository import IBus
from gi.repository import Pango

locale.setlocale(locale.LC_ALL, "")
localedir = os.getenv("IBUS_LOCALEDIR")
pkgdatadir = os.getenv("IBUS_PKGDATADIR") or "."
gettext.install('ibus-libpinyin', localedir)

class KeyboardShortcutSelection(Gtk.Box):
    def __init__(self, shortcut = None):
        super(KeyboardShortcutSelection, self).__init__(
                orientation=Gtk.Orientation.VERTICAL)
        self.__init_ui()
        self.set_shortcut(shortcut)

    def __init_ui(self):
        # shortcut label
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        label = Gtk.Label(_("Shortcut:"))
        label.set_justify(Gtk.Justification.LEFT)
        label.set_alignment(0.0, 0.5)
        hbox.pack_start(label, False, False, 4)

        self.__accel_label = Gtk.Label("")
        self.__accel_label.set_justify(Gtk.Justification.RIGHT)
        self.__accel_label.set_alignment(0.0, 0.5)
        hbox.pack_start(self.__accel_label, False, False, 4)
        self.pack_start(hbox, False, True, 4)

        # key code
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        label = Gtk.Label(label = _("Key code:"))
        label.set_justify(Gtk.Justification.LEFT)
        label.set_alignment(0.0, 0.5)
        hbox.pack_start(label, False, True, 4)

        self.__keycode_entry = Gtk.Entry()
        self.__keycode_entry.connect("notify::text", self.__keycode_entry_notify_cb)
        hbox.pack_start(self.__keycode_entry, True, True, 4)
        self.__keycode_button = Gtk.Button(label = "...")
        self.__keycode_button.connect("clicked", self.__keycode_button_clicked_cb)
        hbox.pack_start(self.__keycode_button, False, True, 4)
        self.pack_start(hbox, False, True, 4)

        # modifiers
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        label = Gtk.Label(label = _("Modifiers:"))
        label.set_justify(Gtk.Justification.LEFT)
        label.set_alignment(0.0, 0.5)
        hbox.pack_start(label, False, True, 4)

        table = Gtk.Table(n_rows = 4, n_columns = 2)
        self.__modifier_buttons = []
        self.__modifier_buttons.append(("Control",
                                        Gtk.CheckButton.new_with_mnemonic("_Control"),
                                        Gdk.ModifierType.CONTROL_MASK))
        self.__modifier_buttons.append(("Alt",
                                        Gtk.CheckButton.new_with_mnemonic("A_lt"),
                                        Gdk.ModifierType.MOD1_MASK))
        self.__modifier_buttons.append(("Shift",
                                        Gtk.CheckButton.new_with_mnemonic("_Shift"),
                                        Gdk.ModifierType.SHIFT_MASK))
        self.__modifier_buttons.append(("Meta",
                                        Gtk.CheckButton.new_with_mnemonic("_Meta"),
                                        Gdk.ModifierType.META_MASK))
        self.__modifier_buttons.append(("Super",
                                        Gtk.CheckButton.new_with_mnemonic("S_uper"),
                                        Gdk.ModifierType.SUPER_MASK))
        self.__modifier_buttons.append(("Hyper",
                                        Gtk.CheckButton.new_with_mnemonic("_Hyper"),
                                        Gdk.ModifierType.HYPER_MASK))
        # <CapsLock> is not parsed by gtk_accelerator_parse()
        # <Release> is not supported by XIGrabKeycode()
        for name, button, mask in self.__modifier_buttons:
            button.connect("toggled", self.__modifier_button_toggled_cb, name)

        table.attach(self.__modifier_buttons[0][1], 0, 1, 0, 1)
        table.attach(self.__modifier_buttons[1][1], 1, 2, 0, 1)
        table.attach(self.__modifier_buttons[2][1], 2, 3, 0, 1)
        table.attach(self.__modifier_buttons[3][1], 0, 1, 1, 2)
        table.attach(self.__modifier_buttons[4][1], 1, 2, 1, 2)
        table.attach(self.__modifier_buttons[5][1], 2, 3, 1, 2)
        hbox.pack_start(table, True, True, 4)
        self.pack_start(hbox, False, True, 4)

    def set_shortcut(self, shortcut = None):
        if shortcut == None:
            shortcut = ""
        self.__accel_label.set_label(shortcut)
        self.__set_shortcut_to_buttons(shortcut)

    def get_shortcut(self):
        return self.__accel_label.get_label()

    def __get_shortcut_from_buttons(self):
        modifiers = []
        keycode = self.__keycode_entry.get_text()
        if Gdk.keyval_from_name(keycode) == 0:
            return None

        for name, button, mask in self.__modifier_buttons:
            if button.get_active():
                modifiers.append(name)
        if keycode.startswith("_"):
            keycode = keycode[1:]
        shortcut = "".join(['<' + m + '>' for m in modifiers])
        shortcut += keycode
        return shortcut

    def __set_shortcut_to_buttons(self, shortcut):
        (keyval, state) = Gtk.accelerator_parse(shortcut)
        if keyval == 0 and state == 0:
            return
        for name, button, mask in self.__modifier_buttons:
            if state & mask:
                button.set_active(True)
            else:
                button.set_active(False)
        self.__keycode_entry.set_text(shortcut.rsplit('>', 1)[-1])

    def __update_accel_label(self):
        shortcut = self.__get_shortcut_from_buttons()
        can_add = shortcut != None
        self.__accel_label.set_label(shortcut)

    def __modifier_button_toggled_cb(self, button, name):
        self.__update_accel_label()

    def __keycode_entry_notify_cb(self, entry, arg):
        self.__update_accel_label()

    def __keycode_button_clicked_cb(self, button):
        out = []
        dlg = Gtk.MessageDialog(transient_for = self.get_toplevel(),
                                buttons = Gtk.ButtonsType.CLOSE)
        message = _("Please press a key (or a key combination).\n" \
                    "The dialog will close when the key is released.")
        dlg.set_markup(message)
        dlg.set_title(_("Please press a key (or a key combination)"))

        def __accel_edited_cb(c, path, keyval, state, keycode):
            out.append(keyval)
            out.append(state)
            out.append(keycode)
            dlg.response(Gtk.ResponseType.OK)

        model = Gtk.ListStore(GObject.TYPE_INT,
                              GObject.TYPE_UINT,
                              GObject.TYPE_UINT)
        accel_view = Gtk.TreeView(model)
        column = Gtk.TreeViewColumn(_("Shorcut Editor"))
        renderer = Gtk.CellRendererAccel(accel_mode=Gtk.CellRendererAccelMode.OTHER,
                                         editable=True)
        renderer.connect('accel-edited', __accel_edited_cb)
        column.pack_start(renderer, True)
        column.add_attribute(renderer, 'accel-mods', 0)
        column.add_attribute(renderer, 'accel-key', 1)
        column.add_attribute(renderer, 'keycode', 2)
        accel_view.append_column(column)
        it = model.append(None)
        area = dlg.get_message_area()
        area.pack_end(accel_view, True, True, 0)
        area.show_all()
        id = dlg.run()
        dlg.destroy()
        if id != Gtk.ResponseType.OK or len(out) < 3:
            return
        keyval = out[0]
        state = out[1]
        keycode = out[2]

        for name, button, mask in self.__modifier_buttons:
            if state & mask:
                button.set_active(True)
            else:
                button.set_active(False)

        shortcut = Gtk.accelerator_name_with_keycode(None,
                                                     keyval,
                                                     keycode,
                                                     state)
        shortcut = shortcut.replace('<Primary>', '<Control>')
        self.__keycode_entry.set_text(shortcut.rsplit('>', 1)[-1])


class KeyboardShortcutSelectionDialog(Gtk.Dialog):
    def __init__(self, title = None, transient_for = None, flags = 0):
        super(KeyboardShortcutSelectionDialog, self).__init__(
                title = title, transient_for = transient_for, flags = flags)
        self.__selection_view = KeyboardShortcutSelection()
        self.vbox.pack_start(self.__selection_view, False, True, 0)
        self.vbox.show_all()

    def set_shortcut(self, shortcut = None):
        self.__selection_view.set_shortcut(shortcut)

    def get_shortcut(self):
        return self.__selection_view.get_shortcut()



if __name__ == "__main__":
    dlg = KeyboardShortcutSelectionDialog(title = "Select test")
    buttons = (_("_Cancel"), Gtk.ResponseType.CANCEL,
               _("_OK"), Gtk.ResponseType.OK)
    dlg.add_buttons(*buttons)
    dlg.set_shortcut("<Shift>")
    dlg.set_shortcut(None)
    print((dlg.run()))
    print((dlg.get_shortcut()))




# vim:set et ts=4 sts=4:
# -*- coding: utf-8 -*-
#
# ibus-libpinyin - Intelligent Pinyin engine based on libpinyin for IBus
#
# Copyright (c) 2012 Peng Wu <alexepico@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gettext
from gi.repository import GObject
from gi.repository import Gtk

gettext.install('ibus-libpinyin')

(
    ART_DICTIONARY,
    CULTURE_DICTIONARY,
    ECONOMY_DICTIONARY,
    GEOLOGY_DICTIONARY,
    HISTORY_DICTIONARY,
    LIFE_DICTIONARY,
    NATURE_DICTIONARY,
    PEOPLE_DICTIONARY,
    SCIENCE_DICTIONARY,
    SOCIETY_DICTIONARY,
    SPORT_DICTIONARY,
    TECHNOLOGY_DICTIONARY,
) = range(4, 16)

(
COLUMN_SENSITIVE,
COLUMN_PHRASE_INDEX,
COLUMN_DESCRIPTION,
COLUMN_ACTIVE
) = range(4)

dictionaries = \
    (
    (True, ART_DICTIONARY, _("Art"), True),
    (True, CULTURE_DICTIONARY, _("Culture"), True),
    (True, ECONOMY_DICTIONARY, _("Economy"), True),
    (True, GEOLOGY_DICTIONARY, _("Geology"), True),
    (True, HISTORY_DICTIONARY, _("History"), True),
    (True, LIFE_DICTIONARY, _("Life"), True),
    (True, NATURE_DICTIONARY, _("Nature"), True),
    (True, PEOPLE_DICTIONARY, _("People"), True),
    (True, SCIENCE_DICTIONARY, _("Science"), True),
    (True, SOCIETY_DICTIONARY, _("Society"), True),
    (True, SPORT_DICTIONARY, _("Sport"), True),
    (True, TECHNOLOGY_DICTIONARY, _("Technology"), True),
    )


class DictionaryTreeView(Gtk.TreeView):
    __gtype_name__ = 'DictionaryTreeView'
    __gproperties__ = {
        'dictionaries': (
            str,
            'dictionaries',
            'Enabled Dictionaries',
            "",
            GObject.PARAM_READWRITE
        )
    }

    def __init__(self):
        super(DictionaryTreeView, self).__init__()

        self.__changed = False

        self.set_headers_visible(True)

        self.__model = self.__create_model()
        self.set_model(self.__model)

        self.__add_columns()

    def __create_model(self):
        model = Gtk.ListStore(bool, int, str, bool)

        model.connect("row-changed", self.__emit_changed, "row-changed")

        for dict in dictionaries:
            iter = model.append()
            model.set(iter,
                      COLUMN_SENSITIVE, dict[COLUMN_SENSITIVE],
                      COLUMN_PHRASE_INDEX, dict[COLUMN_PHRASE_INDEX],
                      COLUMN_DESCRIPTION, dict[COLUMN_DESCRIPTION],
                      COLUMN_ACTIVE, dict[COLUMN_ACTIVE])

        return model

    def __add_columns(self):
        # column for toggles
        renderer = Gtk.CellRendererToggle()
        renderer.connect('toggled', self.__active_toggled, self.__model)
        column = Gtk.TreeViewColumn(_('Active'), renderer, active=COLUMN_ACTIVE, sensitive=COLUMN_SENSITIVE)
        self.append_column(column)

        # column for description
        render = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_('Description'), render, text=COLUMN_DESCRIPTION)
        self.append_column(column)

    def __active_toggled(self, cell, path, model):
        # get toggled iter
        iter = model.get_iter((int(path),))
        active = model.get_value(iter, COLUMN_ACTIVE)

        # toggle active
        active = not active

        # save value
        model.set(iter, COLUMN_ACTIVE, active)

        # notify changed
        self.__changed = True
        self.__emit_changed()

    def __emit_changed(self, *args):
        if self.__changed:
            self.__changed = False
            self.notify("dictionaries")

    def get_dictionaries(self):
        dicts = []
        for row in self.__model:
            if (not row[COLUMN_SENSITIVE]):
                continue;
            if (row[COLUMN_ACTIVE]):
                dicts.append(str(row[COLUMN_PHRASE_INDEX]))

        return ';'.join(dicts)

    def set_dictionaries(self, dicts):
        # clean dictionaries
        for row in self.__model:
            if not row[COLUMN_SENSITIVE]:
                continue
            row[COLUMN_ACTIVE] = False

        if not dicts:
            self.__emit_changed()
            return

        for dict in dicts.split(";"):
            dict = int(dict)
            for row in self.__model:
                if not row[COLUMN_SENSITIVE]:
                    continue
                if dict == row[COLUMN_PHRASE_INDEX]:
                    row[COLUMN_ACTIVE] = True
        self.__emit_changed()

    def do_get_property(self, prop):
        if prop.name == 'dictionaries':
            return self.get_dictionaries()
        else:
            raise AttributeError('unknown property %s' % prop.name)

    def do_set_property(self, prop, value):
        if prop.name == "dictionaries":
            self.set_dictionaries(value)
        else:
            raise AttributeError('unknown property %s' % prop.name)


GObject.type_register(DictionaryTreeView)


if __name__ == "__main__":
    tree = DictionaryTreeView()
    tree.set_dictionaries("")
    w = Gtk.Window()
    w.add(tree)
    w.show_all()
    Gtk.main()



# vim:set et ts=4 sts=4:
# -*- coding: utf-8 -*-
#
# ibus-libpinyin - Intelligent Pinyin engine based on libpinyin for IBus
#
# Copyright (c) 2014 Peng Wu <alexepico@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


__all__ = (
    "ShortcutEditor",
    "ShortcutEditorDialog"
)

import gettext
from gi.repository import GObject
from gi.repository import Gdk
from gi.repository import Gtk

from keyboardshortcut import KeyboardShortcutSelectionDialog

gettext.install('ibus-libpinyin')

(
COLUMN_DESCRIPTION,
COLUMN_CONFIG_KEYNAME,
COLUMN_ACCELERATOR,
) = range(3)

# The default shortcut value is stored here.
accelerators = \
    (
        (_("Switch Chinese/English"), "main-switch" , "<Shift>"),
        (_("Full/Half Width Letter"), "letter-switch", ""),
        (_("Full/Half Width Punct"), "punct-switch", "<Control>period"),
        (_("Full/Half Width Both Switch"), "both-switch", ""),
        (_("Switch Traditional/Simplfied Chinese"), "trad-switch", "<Control><Shift>f")
    )

class ShortcutTreeView(Gtk.TreeView):
    def __init__(self, editor):
        super(ShortcutTreeView, self).__init__()

        self.set_headers_visible(True)

        self.__model = self.__create_model()
        self.set_model(self.__model)

        self.__add_columns()

        self.__editor = editor

    def __create_model(self):
        model = Gtk.ListStore(str, str, str)

        for label, keyname, defvalue in accelerators:
            iter = model.append()
            # (accel_key, accel_mods) = Gtk.accelerator_parse(accel_str)
            model.set(iter,
                      COLUMN_DESCRIPTION, label,
                      COLUMN_CONFIG_KEYNAME, keyname,
                      COLUMN_ACCELERATOR, defvalue,
                      )

        return model

    def __add_columns(self):
        # column for description
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_('Description'), renderer, text=COLUMN_DESCRIPTION)
        self.append_column(column)

        # column for accelerator
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_('Accelerator'), renderer, text=COLUMN_ACCELERATOR)
        self.append_column(column)

    def set_shortcut_value(self, key, value):
        # just clean shortcut
        if value == "":
            for row in self.__model:
                if row[COLUMN_CONFIG_KEYNAME] == key:
                    row[COLUMN_ACCELERATOR] = value
                    return True

        # check duplicate shortcut
        for row in self.__model:
            if row[COLUMN_CONFIG_KEYNAME] == key:
                continue
            if row[COLUMN_ACCELERATOR] == value:
                dialog = Gtk.MessageDialog(None, 0, Gtk.MessageType.WARNING,
                                           Gtk.ButtonsType.OK,
                                           _("This shortcut key is already in use."))
                dialog.run()
                dialog.destroy()
                return False

        # store the shortcut
        for row in self.__model:
            if row[COLUMN_CONFIG_KEYNAME] == key:
                row[COLUMN_ACCELERATOR] = value
                return True

    def set_default_shortcut(self):
        selection = self.get_selection()
        (model, iterator) = selection.get_selected()

        if not iterator:
            return

        key = model[iterator][COLUMN_CONFIG_KEYNAME]
        for label, keyname, defvalue in accelerators:
            if key == keyname:
                if self.set_shortcut_value(key, defvalue):
                    self.__editor.emit_shortcut_changed(key, defvalue)

    def set_shortcut(self, value=""):
        selection = self.get_selection()
        (model, iterator) = selection.get_selected()

        if not iterator:
            return

        key = model[iterator][COLUMN_CONFIG_KEYNAME]
        if self.set_shortcut_value(key, value):
            self.__editor.emit_shortcut_changed(key, value)


class ShortcutEditor(Gtk.Box):
    __gtype_name__ = 'ShortcutEditor'

    __gsignals__ = {
        'shortcut-changed': (GObject.SIGNAL_RUN_FIRST, None,
                             (str, str, ))
    }

    def __init__(self):
        super(ShortcutEditor, self).__init__(
            orientation=Gtk.Orientation.VERTICAL)
        self.__init_ui()

    def __init_ui(self):
        # shortcut tree view
        self.__shortcut_treeview = ShortcutTreeView(self)
        self.__shortcut_treeview.connect("cursor-changed", self.__shortcut_treeview_cursor_changed_cb)
        self.pack_start(self.__shortcut_treeview, False, True, 4)

        # buttons
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        # set default button
        self.__set_default_button = Gtk.Button(label = _("_Default"),
                                               use_underline = True)
        self.__set_default_button.connect("clicked", self.__set_default_button_clicked_cb)
        hbox.pack_start(self.__set_default_button, False, True, 0)
        # edit button
        self.__edit_button = Gtk.Button(label= _("_Edit"),
                                        use_underline = True)
        self.__edit_button.connect("clicked", self.__edit_button_clicked_cb)
        hbox.pack_start(self.__edit_button, False, True, 0)
        self.pack_start(hbox, False, True, 4)
        # select the first row
        selection = self.__shortcut_treeview.get_selection()
        selection.set_mode(Gtk.SelectionMode.SINGLE)
        selection.select_path(Gtk.TreePath(0))
        self.show_all()

    def __shortcut_treeview_cursor_changed_cb(self, treeview):
        selection = treeview.get_selection()
        (model, iterator) = selection.get_selected()

        if iterator:
            self.__set_default_button.set_sensitive(True)
            self.__edit_button.set_sensitive(True)
        else:
            self.__set_default_button.set_sensitive(False)
            self.__edit_button.set_sensitive(False)

    def __set_default_button_clicked_cb(self, button):
        self.__shortcut_treeview.set_default_shortcut()

    def __edit_button_clicked_cb(self, button):
        dlg = KeyboardShortcutSelectionDialog(title = _("Select Switching Key"))
        buttons = (_("_Cancel"), Gtk.ResponseType.CANCEL,
                   _("_OK"), Gtk.ResponseType.OK)
        dlg.add_buttons(*buttons)

        selection = self.__shortcut_treeview.get_selection()
        (model, iterator) = selection.get_selected()
        if iterator:
            dlg.set_shortcut(model[iterator][COLUMN_ACCELERATOR])

        response = dlg.run()
        dlg.destroy()
        if response == Gtk.ResponseType.CANCEL:
            return

        self.__shortcut_treeview.set_shortcut(dlg.get_shortcut())

    def emit_shortcut_changed(self, key, value):
        self.emit("shortcut-changed", key, value)

    def update_shortcuts(self, config):
        for label, keyname, defvalue in accelerators:
            value = config.get_value(keyname).get_string()
            self.__shortcut_treeview.set_shortcut_value(keyname, value)
            # store the default value
            # self.emit_shortcut_changed(keyname, value)

class ShortcutEditorDialog(Gtk.Dialog):
    def __init__(self, title = None, transient_for = None, flags = 0):
        super(ShortcutEditorDialog, self).__init__(
                title = title, transient_for = transient_for, flags = flags)
        self.__shortcut_editor = ShortcutEditor()
        self.vbox.pack_start(self.__shortcut_editor, False, True, 0)
        self.vbox.show_all()


if __name__ == "__main__":
    dlg = ShortcutEditorDialog(title = "Shortcut Editor test")
    buttons = (_("_Cancel"), Gtk.ResponseType.CANCEL,
               _("_OK"), Gtk.ResponseType.OK)
    dlg.add_buttons(*buttons)
    print((dlg.run()))



# vim:set et ts=4 sts=4:
# -*- coding: utf-8 -*-
#
# ibus-libpinyin - Intelligent Pinyin engine based on libpinyin for IBus
#
# Copyright (c) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
# Copyright (c) 2010 BYVoid <byvoid1@gmail.com>
# Copyright (c) 2011-2024 Peng Wu <alexepico@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gettext
from enginefile import resync_engine_file, save_layout

import locale
import os
import sys

from gi import require_version as gi_require_version
gi_require_version('GLib', '2.0')
gi_require_version('Gio', '2.0')
gi_require_version('Gtk', '3.0')
gi_require_version('IBus', '1.0')

from gi.repository import GLib
from gi.repository import Gio

# set_prgname before importing other modules to show the name in warning
# messages when import modules are failed. E.g. Gtk.
GLib.set_prgname('ibus-setup-libpinyin')

from gi.repository import Gtk
from gi.repository import IBus

import config
from dicttreeview import DictionaryTreeView
from shortcuteditor import ShortcutEditor

DOMAINNAME = 'ibus-libpinyin'
locale.setlocale(locale.LC_ALL, "")
localedir = os.getenv("IBUS_LOCALEDIR")
pkgdatadir = os.getenv("IBUS_PKGDATADIR") or "."

# Python's locale module doesn't provide all methods on some
# operating systems like FreeBSD
try:
    locale.bindtextdomain(DOMAINNAME, localedir)
    locale.bind_textdomain_codeset(DOMAINNAME, 'UTF-8')
except AttributeError:
    pass

# Python's gettext module doesn't provide all methods in
# new Python version
try:
    gettext.bindtextdomain(DOMAINNAME, localedir)
    gettext.bind_textdomain_codeset(DOMAINNAME, 'UTF-8')
except AttributeError:
    pass

gettext.install(DOMAINNAME, localedir)

class PreferencesDialog:
    def __init__(self, engine):
        self.__bus = IBus.Bus()
        self.__builder = Gtk.Builder()
        self.__builder.set_translation_domain(DOMAINNAME)
        self.__builder.add_from_file("ibus-libpinyin-preferences.ui")
        self.__dialog = self.__builder.get_object("dialog")
        self.__init_pages()

        if engine == "libpinyin":
            self.__config_namespace = "com.github.libpinyin.ibus-libpinyin.libpinyin"
            self.__config = Gio.Settings.new(self.__config_namespace)
            self.__init_general()
            self.__init_pinyin()
            self.__init_fuzzy()
            self.__init_dictionary()
            self.__init_user_data()
            self.__init_shortcut()
            self.__init_cloud_input()
            self.__init_about()
        elif engine == "libbopomofo":
            self.__config_namespace = "com.github.libpinyin.ibus-libpinyin.libbopomofo"
            self.__config = Gio.Settings.new(self.__config_namespace)
            self.__init_general()
            self.__init_bopomofo()
            self.__init_fuzzy()
            self.__init_dictionary()
            #self.__init_user_data()
            self.__init_shortcut()
            self.__init_bopomofo_cloud_input()
            self.__init_about()
            self.__convert_fuzzy_pinyin_to_bopomofo()
            self.__display_style_box.hide()

        else:
            print("Error: Unknown Engine")
            exit()

        self.__pages.set_current_page(0)

    def __init_pages(self):
        self.__pages = self.__builder.get_object("pages")
        self.__page_general = self.__builder.get_object("pageGeneral")
        self.__page_pinyin_mode = self.__builder.get_object("pagePinyinMode")
        self.__page_bopomofo_mode = self.__builder.get_object("pageBopomofoMode")
        self.__page_fuzzy = self.__builder.get_object("pageFuzzy")
        self.__page_dictionary = self.__builder.get_object("pageDictionary")
        self.__page_user_data = self.__builder.get_object("pageUserData")
        self.__page_shortcut = self.__builder.get_object("pageShortcut")
        self.__page_about = self.__builder.get_object("pageAbout")

        self.__page_general.hide()
        self.__page_pinyin_mode.hide()
        self.__page_bopomofo_mode.hide()
        self.__page_fuzzy.hide()
        self.__page_dictionary.hide()
        self.__page_user_data.hide()
        self.__page_about.hide()

    def __init_general(self):
        # page General
        self.__page_general.show()

        # init state
        self.__init_chinese = self.__builder.get_object("InitChinese")
        self.__init_english = self.__builder.get_object("InitEnglish")
        self.__init_full = self.__builder.get_object("InitFull")
        self.__init_half = self.__builder.get_object("InitHalf")
        self.__init_full_punct = self.__builder.get_object("InitFullPunct")
        self.__init_half_punct = self.__builder.get_object("InitHalfPunct")
        self.__init_simp = self.__builder.get_object("InitSimplifiedChinese")
        self.__init_trad = self.__builder.get_object("InitTraditionalChinese")

        # UI
        self.__display_style = self.__builder.get_object("DisplayStyle")
        self.__display_style_box = self.__builder.get_object("boxDisplayStyle")
        self.__lookup_table_orientation = self.__builder.get_object("LookupTableOrientation")
        self.__lookup_table_page_size = self.__builder.get_object("LookupTablePageSize")

        self.__dynamic_adjust = self.__builder.get_object("DynamicAdjust")
        self.__remember_every_input = self.__builder.get_object("RememberEveryInput")
        self.__sort_candidate_option = self.__builder.get_object("SortCandidateOption")

        self.__keyboard_layout = self.__builder.get_object("KeyboardLayout")
        self.__keyboard_layout_information = self.__builder.get_object("KeyboardLayoutInformation")

        # read values
        self.__init_chinese.set_active(self.__get_value("init-chinese"))
        self.__init_full.set_active(self.__get_value("init-full"))
        self.__init_full_punct.set_active(self.__get_value("init-full-punct"))
        self.__init_simp.set_active(self.__get_value("init-simplified-chinese"))

        self.__lookup_table_orientation.set_active(self.__get_value("lookup-table-orientation"))
        self.__lookup_table_page_size.set_value(self.__get_value("lookup-table-page-size"))
        self.__display_style.set_active(self.__get_value("display-style"))

        self.__dynamic_adjust.set_active(self.__get_value("dynamic-adjust"))
        self.__remember_every_input.set_active(self.__get_value("remember-every-input"))
        self.__sort_candidate_option.set_active(self.__get_value("sort-candidate-option"))

        self.__keyboard_layout.set_active_id(self.__get_value("keyboard-layout"))

        # connect signals
        self.__init_chinese.connect("toggled", self.__toggled_cb, "init-chinese")
        self.__init_full.connect("toggled", self.__toggled_cb, "init-full")
        self.__init_full_punct.connect("toggled", self.__toggled_cb, "init-full-punct")
        self.__init_simp.connect("toggled", self.__toggled_cb, "init-simplified-chinese")
        self.__dynamic_adjust.connect("toggled", self.__toggled_cb, "dynamic-adjust")
        self.__remember_every_input.connect("toggled", self.__toggled_cb, "remember-every-input")

        def __display_size_changed_cb(widget):
            self.__set_value("display-style", widget.get_active())

        def __lookup_table_orientation_changed_cb(widget):
            self.__set_value("lookup-table-orientation", widget.get_active())

        def __lookup_table_page_size_changed_cb(adjustment):
            self.__set_value("lookup-table-page-size", int(adjustment.get_value()))

        def __sort_candidate_option_changed_cb(widget):
            self.__set_value("sort-candidate-option", widget.get_active())

        def __keyboard_layout_changed_cb(widget):
            self.__set_value("keyboard-layout", widget.get_active_id())
            save_layout()
            self.__keyboard_layout_information.show()

        self.__display_style.connect("changed", __display_size_changed_cb)
        self.__lookup_table_orientation.connect("changed", __lookup_table_orientation_changed_cb)
        self.__lookup_table_page_size.connect("value-changed", __lookup_table_page_size_changed_cb)
        self.__sort_candidate_option.connect("changed", __sort_candidate_option_changed_cb)
        self.__keyboard_layout.connect("changed", __keyboard_layout_changed_cb)

    def __init_pinyin(self):
        # page
        self.__page_pinyin_mode.show()

        # pinyin
        self.__full_pinyin = self.__builder.get_object("FullPinyin")
        self.__incomplete_pinyin = self.__builder.get_object("IncompletePinyin")
        self.__double_pinyin = self.__builder.get_object("DoublePinyin")
        self.__double_pinyin_schema = self.__builder.get_object("DoublePinyinSchema")
        # self.__double_pinyin_schema_label = self.__builder.get_object("labelDoublePinyinSchema")
        self.__double_pinyin_show_raw = self.__builder.get_object("DoublePinyinShowRaw")

        # read value
        self.__incomplete_pinyin.set_active(self.__get_value("incomplete-pinyin"))
        self.__full_pinyin.set_active(not self.__get_value("double-pinyin"))
        self.__double_pinyin_schema.set_active(self.__get_value("double-pinyin-schema"))
        self.__double_pinyin_show_raw.set_active(self.__get_value("double-pinyin-show-raw"))
        if self.__full_pinyin.get_active():
            # self.__incomplete_pinyin.set_sensitive(True)
            self.__double_pinyin_schema.set_sensitive(False)
            # self.__double_pinyin_schema_label.set_sensitive(False)
            self.__double_pinyin_show_raw.set_sensitive(False)
        else:
            # self.__incomplete_pinyin.set_sensitive(False)
            self.__double_pinyin_schema.set_sensitive(True)
            # self.__double_pinyin_schema_label.set_sensitive(True)
            self.__double_pinyin_show_raw.set_sensitive(True)

        def __double_pinyin_toggled_cb(widget):
            val = widget.get_active()
            self.__set_value("double-pinyin", val)
            self.__double_pinyin_schema.set_sensitive(val)
            # self.__double_pinyin_schema_label.set_sensitive(val)
            self.__double_pinyin_show_raw.set_sensitive(val)

        def __double_pinyin_schema_changed_cb(widget):
            self.__set_value("double-pinyin-schema", widget.get_active())

        # connect signals
        self.__double_pinyin.connect("toggled", __double_pinyin_toggled_cb)
        self.__incomplete_pinyin.connect("toggled", self.__toggled_cb, "incomplete-pinyin")
        self.__double_pinyin_schema.connect("changed", __double_pinyin_schema_changed_cb)
        self.__double_pinyin_show_raw.connect("toggled", self.__toggled_cb, "double-pinyin-show-raw")

        self.__init_input_custom()
        self.__init_correct_pinyin()

    def __init_bopomofo(self):
        # page Bopomodo Mode
        self.__page_bopomofo_mode.show()

        # bopomofo mode
        self.__incomplete_bopomofo = self.__builder.get_object("IncompleteBopomofo")
        self.__bopomofo_keyboard_mapping = self.__builder.get_object("BopomofoKeyboardMapping")

        # selection mode
        self.__select_keys = self.__builder.get_object("SelectKeys")
        self.__guide_key = self.__builder.get_object("GuideKey")
        self.__auxiliary_select_key_f = self.__builder.get_object("AuxiliarySelectKey_F")
        self.__auxiliary_select_key_kp = self.__builder.get_object("AuxiliarySelectKey_KP")

        # other
        self.__enter_key = self.__builder.get_object("CommitFirstCandidate")

        # read value
        self.__bopomofo_keyboard_mapping.set_active(self.__get_value("bopomofo-keyboard-mapping"))
        self.__incomplete_bopomofo.set_active(self.__get_value("incomplete-pinyin"))
        self.__select_keys.set_active(self.__get_value("select-keys"))
        self.__guide_key.set_active(self.__get_value("guide-key"))
        self.__auxiliary_select_key_f.set_active(self.__get_value("auxiliary-select-key-f"))
        self.__auxiliary_select_key_kp.set_active(self.__get_value("auxiliary-select-key-kp"))
        self.__enter_key.set_active(self.__get_value("enter-key"))

        # connect signals
        def __bopomofo_keyboard_mapping_changed_cb(widget):
            self.__set_value("bopomofo-keyboard-mapping", widget.get_active())
        def __select_keys_changed_cb(widget):
            self.__set_value("select-keys", widget.get_active())

        self.__bopomofo_keyboard_mapping.connect("changed", __bopomofo_keyboard_mapping_changed_cb)
        self.__incomplete_bopomofo.connect("toggled", self.__toggled_cb, "incomplete-pinyin")
        self.__select_keys.connect("changed", __select_keys_changed_cb)
        self.__guide_key.connect("toggled", self.__toggled_cb, "guide-key")
        self.__auxiliary_select_key_f.connect("toggled", self.__toggled_cb, "auxiliary-select-key-f")
        self.__auxiliary_select_key_kp.connect("toggled", self.__toggled_cb, "auxiliary-select-key-kp")
        self.__enter_key.connect("toggled", self.__toggled_cb, "enter-key")

    def __init_input_custom(self):
        # others
        self.__shift_select_candidate = self.__builder.get_object("ShiftSelectCandidate")
        self.__minus_equal_page = self.__builder.get_object("MinusEqualPage")
        self.__comma_period_page = self.__builder.get_object("CommaPeriodPage")
        self.__square_bracket_page = self.__builder.get_object("SquareBracketPage")
        self.__auto_commit = self.__builder.get_object("AutoCommit")

        # read values
        self.__shift_select_candidate.set_active(self.__get_value("shift-select-candidate"))
        self.__minus_equal_page.set_active(self.__get_value("minus-equal-page"))
        self.__comma_period_page.set_active(self.__get_value("comma-period-page"))
        self.__square_bracket_page.set_active(self.__get_value("square-bracket-page"))
        self.__auto_commit.set_active(self.__get_value("auto-commit"))

        # connect signals
        self.__shift_select_candidate.connect("toggled", self.__toggled_cb, "shift-select-candidate")
        self.__minus_equal_page.connect("toggled", self.__toggled_cb, "minus-equal-page")
        self.__comma_period_page.connect("toggled", self.__toggled_cb, "comma-period-page")
        self.__square_bracket_page.connect("toggled", self.__toggled_cb, "square-bracket-page")
        self.__auto_commit.connect("toggled", self.__toggled_cb, "auto-commit")

    def __init_correct_pinyin(self):
        # auto correct
        self.__correct_pinyin = self.__builder.get_object("CorrectPinyin")
        self.__correct_pinyin_widgets = [
            ('CorrectPinyin_GN_NG', 'correct-pinyin-gn-ng'),
            ('CorrectPinyin_MG_NG', 'correct-pinyin-mg-ng'),
            ('CorrectPinyin_IOU_IU', 'correct-pinyin-iou-iu'),
            ('CorrectPinyin_UEI_UI', 'correct-pinyin-uei-ui'),
            ('CorrectPinyin_UEN_UN', 'correct-pinyin-uen-un'),
            ('CorrectPinyin_UE_VE', 'correct-pinyin-ue-ve'),
            ('CorrectPinyin_V_U', 'correct-pinyin-v-u'),
            ('CorrectPinyin_ON_ONG', 'correct-pinyin-on-ong'),
        ]

        def __correct_pinyin_toggled_cb(widget):
            val = widget.get_active()
            for w in self.__correct_pinyin_widgets:
                self.__builder.get_object(w[0]).set_sensitive(val)
        self.__correct_pinyin.connect("toggled", __correct_pinyin_toggled_cb)

        # init value
        self.__correct_pinyin.set_active(self.__get_value("correct-pinyin"))
        for name, keyname in self.__correct_pinyin_widgets:
            widget = self.__builder.get_object(name)
            widget.set_active(self.__get_value(keyname))

        self.__correct_pinyin.connect("toggled", self.__toggled_cb, "correct-pinyin")
        for name, keyname in self.__correct_pinyin_widgets:
            widget = self.__builder.get_object(name)
            widget.connect("toggled", self.__toggled_cb, keyname)

    def __init_fuzzy(self):
        # page Fuzzy
        self.__page_fuzzy.show()

        # fuzzy pinyin
        self.__fuzzy_pinyin = self.__builder.get_object("FuzzyPinyin")
        self.__fuzzy_pinyin_widgets = [
            ('FuzzyPinyin_C_CH', 'fuzzy-pinyin-c-ch'),
            ('FuzzyPinyin_Z_ZH', 'fuzzy-pinyin-z-zh'),
            ('FuzzyPinyin_S_SH', 'fuzzy-pinyin-s-sh'),
            ('FuzzyPinyin_L_N', 'fuzzy-pinyin-l-n'),
            ('FuzzyPinyin_F_H', 'fuzzy-pinyin-f-h'),
            ('FuzzyPinyin_L_R', 'fuzzy-pinyin-l-r'),
            ('FuzzyPinyin_G_K', 'fuzzy-pinyin-g-k'),
            ('FuzzyPinyin_AN_ANG', 'fuzzy-pinyin-an-ang'),
            ('FuzzyPinyin_EN_ENG', 'fuzzy-pinyin-en-eng'),
            ('FuzzyPinyin_IN_ING', 'fuzzy-pinyin-in-ing'),
        ]

        def __fuzzy_pinyin_toggled_cb(widget):
            val = widget.get_active()
            for w in self.__fuzzy_pinyin_widgets:
                self.__builder.get_object(w[0]).set_sensitive(val)
        self.__fuzzy_pinyin.connect("toggled", __fuzzy_pinyin_toggled_cb)

        # read values
        self.__fuzzy_pinyin.set_active(self.__get_value("fuzzy-pinyin"))
        for name, keyname in self.__fuzzy_pinyin_widgets:
            widget = self.__builder.get_object(name)
            widget.set_active(self.__get_value(keyname))

        # connect signals
        self.__fuzzy_pinyin.connect("toggled", self.__toggled_cb, "fuzzy-pinyin")
        for name, keyname in self.__fuzzy_pinyin_widgets:
            widget = self.__builder.get_object(name)
            widget.connect("toggled", self.__toggled_cb, keyname)

    def __convert_fuzzy_pinyin_to_bopomofo(self):
        options = [
            ("FuzzyPinyin_C_CH",   "ㄘ <=> ㄔ"),
            ("FuzzyPinyin_Z_ZH",   "ㄗ <=> ㄓ"),
            ("FuzzyPinyin_S_SH",   "ㄙ <=> ㄕ"),
            ("FuzzyPinyin_L_N",    "ㄌ <=> ㄋ"),
            ("FuzzyPinyin_F_H",    "ㄈ <=> ㄏ"),
            ("FuzzyPinyin_L_R",    "ㄌ <=> ㄖ"),
            ("FuzzyPinyin_G_K",    "ㄍ <=> ㄎ"),
            ("FuzzyPinyin_AN_ANG", "ㄢ <=> ㄤ"),
            ("FuzzyPinyin_EN_ENG", "ㄣ <=> ㄥ"),
            ("FuzzyPinyin_IN_ING", "ㄧㄣ <=> ㄧㄥ"),
        ]

        for name, label in options:
            self.__builder.get_object(name).set_label(label)


    def __init_dictionary(self):
        # page Dictionary
        self.__page_dictionary.show()

        # dictionary tree view
        self.__dict_treeview = self.__builder.get_object("Dictionaries")
        self.__dict_treeview.show()
        self.__dict_treeview.set_dictionaries(self.__get_value("dictionaries"))

        self.__dictionary_information = self.__builder.get_object("DictionaryInformation")

        def __notified_dicts_cb(self, param, dialog):
            dialog.__set_value("dictionaries", self.get_dictionaries())
            dialog.__dictionary_information.show()

        # connect notify signal
        self.__dict_treeview.connect("notify::dictionaries", __notified_dicts_cb, self)

    def __init_user_data(self):
        #page User Data
        self.__page_user_data.show()
        self.__frame_input_modes = self.__builder.get_object("frameInputModes")
        grid = self.__builder.get_object("gridInputModes")

        path = os.path.join(pkgdatadir, 'user.lua')
        if os.access(path, os.R_OK):
            lua_visible = config.enable_lua_extension()
        else:
            lua_visible = False
        table_mode_visible = config.enable_table_mode()
        english_mode_visible = config.enable_english_input_mode()

        input_mode_buttons = [
#            obj_id,                 label,                  visible,              key,                    callback_func,           func_param;
            ("lua_extension",        "Lua Extension",        lua_visible,          "lua-extension",        self.__lua_extension_cb, None),
            ("table_mode",           "Table Mode",           table_mode_visible,   "table-input-mode",     self.__table_mode_cb,    None),
            ("english_mode",         "English Mode",         english_mode_visible, "english-input-mode",   self.__english_mode_cb,  None),
            ("english_candidate",    "English Candidate",    True,                 "english-candidate",    self.__toggled_cb,       "english-candidate"),
            ("emoji_candidate",      "Emoji Candidate",      True,                 "emoji-candidate",      self.__toggled_cb,       "emoji-candidate"),
            ("suggestion_candidate", "Suggestion Candidate", True,                 "suggestion-candidate", self.__toggled_cb,       "suggestion-candidate"),
        ]

        row = 0
        col = 0
        self.input_mode_buttons = {}
        for obj_id, label, visible, key, callback_func, func_param in input_mode_buttons:
            if not visible:
                continue
            else:
                # Button initial:
                button = Gtk.CheckButton(label=label)
                button.set_name(obj_id)
                button.set_visible(True)
                button.set_can_focus(True)
                button.set_receives_default(False)
                button.set_halign(Gtk.Align.START)
                grid.attach(button, col, row, 1, 1)
                button.set_active(self.__get_value(key))
                if func_param is None:
                    button.connect("toggled", callback_func)
                else:
                    button.connect("toggled", callback_func, func_param)

            # Move to the next grid position:
            col += 1
            if col > 1:
                col = 0
                row += 1

        self.__frame_lua_script = self.__builder.get_object("frameLuaScript")
        if lua_visible:
            self.__frame_lua_script.set_sensitive(self.__get_value("lua-extension"))
            self.__edit_lua = self.__builder.get_object("EditLua")
            self.__edit_lua.connect("clicked", self.__edit_lua_cb)
        else:
            self.__frame_lua_script.hide()

        self.__frame_user_dictionary = self.__builder.get_object("frameUserDictionary")
        self.__export_dictionary = self.__builder.get_object("ExportDictionary")
        self.__export_dictionary.connect("clicked", self.__export_dictionary_cb, "export-dictionary")
        self.__import_dictionary = self.__builder.get_object("ImportDictionary")
        self.__import_dictionary.connect("clicked", self.__import_dictionary_cb, "import-dictionary")
        self.__clear_all_data = self.__builder.get_object("ClearAllDictionary")
        self.__clear_all_data.connect("clicked", self.__clear_user_data_cb, "all")
        self.__clear_user_data = self.__builder.get_object("ClearUserDictionary")
        self.__clear_user_data.connect("clicked", self.__clear_user_data_cb, "user")

        self.__frame_user_table = self.__builder.get_object("frameUserTable")
        if not config.enable_table_mode():
            self.__frame_user_table.hide()
        else:
            self.__frame_user_table.set_sensitive(self.__get_value("table-input-mode"))
            self.__export_table = self.__builder.get_object("ExportTable")
            self.__export_table.connect("clicked", self.__export_table_cb, "export-custom-table")
            self.__import_table = self.__builder.get_object("ImportTable")
            self.__import_table.connect("clicked", self.__import_table_cb, "import-custom-table")
            self.__clear_user_table = self.__builder.get_object("ClearUserTable")
            self.__clear_user_table.connect("clicked", self.__clear_user_table_cb, "clear-custom-table", "user")

    def __lua_extension_cb(self, widget):
        self.__set_value("lua-extension", widget.get_active())
        self.__frame_lua_script.set_sensitive(widget.get_active())

    def __table_mode_cb(self, widget):
        self.__set_value("table-input-mode", widget.get_active())
        self.__frame_user_table.set_sensitive(widget.get_active())

    def __english_mode_cb(self, widget):
        self.__set_value("english-input-mode", widget.get_active())

    def __edit_lua_cb(self, widget):
        import shutil
        path = os.path.join(GLib.get_user_config_dir(), "ibus", "libpinyin")
        os.path.exists(path) or os.makedirs(path)
        path = os.path.join(path, "user.lua")
        if not os.path.exists(path):
            src = os.path.join(pkgdatadir, "user.lua")
            shutil.copyfile(src, path)
        os.system("xdg-open %s" % path)

    def __get_import_filename(self):
        dialog = Gtk.FileChooserDialog \
            (title = _("Please choose a file"), parent = self.__dialog,
             action = Gtk.FileChooserAction.OPEN)

        dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                           Gtk.STOCK_OPEN, Gtk.ResponseType.OK)

        dialog.set_current_folder(os.path.expanduser("~"))

        filter_text = Gtk.FileFilter()
        filter_text.set_name("Text files")
        filter_text.add_mime_type("text/plain")
        dialog.add_filter(filter_text)

        filename = None
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            filename = dialog.get_filename()
        dialog.destroy()

        return filename

    def __get_export_filename(self):
        dialog = Gtk.FileChooserDialog \
                 (title = _("Please save a file"), parent = self.__dialog,
                  action = Gtk.FileChooserAction.SAVE)

        dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                           Gtk.STOCK_SAVE, Gtk.ResponseType.OK)

        dialog.set_current_folder(os.path.expanduser("~"))

        dialog.set_do_overwrite_confirmation(True)

        filter_text = Gtk.FileFilter()
        filter_text.set_name("Text files")
        filter_text.add_mime_type("text/plain")
        dialog.add_filter(filter_text)

        filename = None
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            filename = dialog.get_filename()
        dialog.destroy()

        return filename

    def __import_dictionary_cb(self, widget, name):
        filename = self.__get_import_filename()
        if filename:
            self.__set_value(name, "")
            self.__set_value(name, filename)

    def __export_dictionary_cb(self, widget, name):
        filename = self.__get_export_filename()
        if filename:
            self.__set_value(name, "")
            self.__set_value(name, filename)

    def __clear_user_data_cb(self, widget, name):
        self.__set_value("clear-user-data", name)

    def __import_table_cb(self, widget, name):
        filename = self.__get_import_filename()
        if filename:
            self.__set_value(name, "")
            self.__set_value(name, filename)
            self.__set_value("use-custom-table", True)

    def __export_table_cb(self, widget, name):
        filename = self.__get_export_filename()
        if filename:
            self.__set_value(name, "")
            self.__set_value(name, filename)

    def __clear_user_table_cb(self, widget, name, value):
        self.__set_value(name, value)
        self.__set_value("use-custom-table", False)

    def __init_shortcut(self):
        # page Shortcut
        self.__page_shortcut.show()

        # shortcut tree view
        self.__shortcut_editor = self.__builder.get_object("ShortcutsEditor")
        # work around for fedora 21
        self.__shortcut_editor.set_orientation(Gtk.Orientation.VERTICAL)
        self.__shortcut_editor.show()

        # connect "shortcut-changed" signal
        self.__shortcut_editor.connect("shortcut-changed", self.__shortcut_changed_cb)

        # set shortcuts
        self.__shortcut_editor.update_shortcuts(self.__config)

    def __shortcut_changed_cb(self, editor, key, value):
        self.__set_value(key, value)

    def __init_bopomofo_cloud_input(self):
        # init state
        self.__frame_cloud_input = self.__builder.get_object("frameBopomofoCloudInput")
        self.__init_enable_cloud_input = self.__builder.get_object("InitEnableBopomofoCloudInput")

        # cloud input option
        self.__cloud_input_source = self.__builder.get_object("BopomofoCloudInputSource")

        if not config.support_cloud_input():
            self.__frame_cloud_input.hide()

        # read values
        self.__init_enable_cloud_input.set_active(self.__get_value("enable-cloud-input"))

        self.__cloud_input_source.set_active(self.__get_value("cloud-input-source"))

        if self.__init_enable_cloud_input.get_active():
            self.__cloud_input_source.set_sensitive(True)
        else:
            self.__cloud_input_source.set_sensitive(False)

        # connect signals
        def __enable_cloud_input_cb(widget):
            val = widget.get_active()
            self.__set_value("enable-cloud-input", val)
            self.__cloud_input_source.set_sensitive(val)

        def __cloud_input_source_changed_cb(widget):
            self.__set_value("cloud-input-source", widget.get_active())

        self.__init_enable_cloud_input.connect("toggled", __enable_cloud_input_cb)
        self.__cloud_input_source.connect("changed", __cloud_input_source_changed_cb)


    def __init_cloud_input(self):
        # init state
        self.__frame_cloud_input = self.__builder.get_object("frameCloudInput")
        self.__init_enable_cloud_input = self.__builder.get_object("InitEnableCloudInput")

        # cloud input option
        self.__cloud_input_source = self.__builder.get_object("CloudInputSource")

        if not config.support_cloud_input():
            self.__frame_cloud_input.hide()

        # read values
        self.__init_enable_cloud_input.set_active(self.__get_value("enable-cloud-input"))

        self.__cloud_input_source.set_active(self.__get_value("cloud-input-source"))

        if self.__init_enable_cloud_input.get_active():
            self.__cloud_input_source.set_sensitive(True)
        else:
            self.__cloud_input_source.set_sensitive(False)

        # connect signals
        def __enable_cloud_input_cb(widget):
            val = widget.get_active()
            self.__set_value("enable-cloud-input", val)
            self.__cloud_input_source.set_sensitive(val)

        def __cloud_input_source_changed_cb(widget):
            self.__set_value("cloud-input-source", widget.get_active())

        self.__init_enable_cloud_input.connect("toggled", __enable_cloud_input_cb)
        self.__cloud_input_source.connect("changed", __cloud_input_source_changed_cb)

    def __init_about(self):
        # page About
        self.__page_about.show()

        self.__about_icon = self.__builder.get_object("AboutIcon")
        icon_path = os.path.join(pkgdatadir, 'icons', 'ibus-pinyin.svg')
        self.__about_icon.set_from_file(icon_path)

        self.__name_version = self.__builder.get_object("NameVersion")
        self.__name_version.set_markup(_("<big><b>Intelligent Pinyin %s</b></big>") % config.get_version())

    def __changed_cb(self, widget, name):
        self.__set_value(name, widget.get_active())

    def __toggled_cb(self, widget, name):
        self.__set_value(name, widget.get_active ())

    def __get_value(self, name):
        var = self.__config.get_value(name)
        vartype = var.get_type_string()
        if vartype == 'b':
            return var.get_boolean()
        elif vartype == 'i':
            return var.get_int32()
        elif vartype == 's':
            return var.get_string()
        else:
            print("var(%s) is not in support type." % repr(var), file=sys.stderr)
            return None

    def __set_value(self, name, val):
        var = None
        if isinstance(val, bool):
            var = GLib.Variant.new_boolean(val)
        elif isinstance(val, int):
            var = GLib.Variant.new_int32(val)
        elif isinstance(val, str):
            var = GLib.Variant.new_string(val)
        else:
            print("val(%s) is not in support type." % repr(val), file=sys.stderr)
            return

        self.__config.set_value(name, var)

    def run(self):
        return self.__dialog.run()

def main():
    command_name = "libpinyin"
    if len(sys.argv) == 2:
        command_name = sys.argv[1]

    resync_engine_file()
    if command_name == "resync-engine":
        return

    if command_name not in ("libpinyin", "libbopomofo"):
        command_name = "libpinyin"

    PreferencesDialog(command_name).run()


if __name__ == "__main__":
    main()
